@@ -9,10 +9,18 @@
 
 **********************************************************************/
 
-#include "internal.h"
-#include "vm_core.h"
 #include "id.h"
+#include "internal.h"
+#include "internal/class.h"
+#include "internal/error.h"
+#include "internal/hash.h"
+#include "internal/object.h"
+#include "internal/proc.h"
+#include "internal/struct.h"
+#include "internal/symbol.h"
 #include "transient_heap.h"
+#include "vm_core.h"
+#include "builtin.h"
 
 /* only for struct[:field] access */
 enum {
@@ -20,8 +28,8 @@ enum {
     AREF_HASH_THRESHOLD = 10
 };
 
-const rb_iseq_t *rb_method_for_self_aref(VALUE name, VALUE arg, rb_insn_func_t func);
-const rb_iseq_t *rb_method_for_self_aset(VALUE name, VALUE arg, rb_insn_func_t func);
+const rb_iseq_t *rb_method_for_self_aref(VALUE name, VALUE arg, const struct rb_builtin_function *func);
+const rb_iseq_t *rb_method_for_self_aset(VALUE name, VALUE arg, const struct rb_builtin_function *func);
 
 VALUE rb_cStruct;
 static ID id_members, id_back_members, id_keyword_init;
@@ -281,11 +289,42 @@ new_struct(VALUE name, VALUE super)
     return rb_define_class_id_under(super, id, super);
 }
 
+NORETURN(static void invalid_struct_pos(VALUE s, VALUE idx));
+
+static inline long
+struct_pos_num(VALUE s, VALUE idx)
+{
+    long i = NUM2INT(idx);
+    if (i < 0 || i >= RSTRUCT_LEN(s)) invalid_struct_pos(s, idx);
+    return i;
+}
+
+static VALUE
+opt_struct_aref(rb_execution_context_t *ec, VALUE self, VALUE idx)
+{
+    long i = struct_pos_num(self, idx);
+    return RSTRUCT_GET(self, i);
+}
+
+static VALUE
+opt_struct_aset(rb_execution_context_t *ec, VALUE self, VALUE val, VALUE idx)
+{
+    long i = struct_pos_num(self, idx);
+    rb_struct_modify(self);
+    RSTRUCT_SET(self, i, val);
+    return val;
+}
+
+static const struct rb_builtin_function struct_aref_builtin =
+    RB_BUILTIN_FUNCTION(0, struct_aref, opt_struct_aref, 1, 0);
+static const struct rb_builtin_function struct_aset_builtin =
+    RB_BUILTIN_FUNCTION(1, struct_aref, opt_struct_aset, 2, 0);
+
 static void
 define_aref_method(VALUE nstr, VALUE name, VALUE off)
 {
-    rb_control_frame_t *FUNC_FASTCALL(rb_vm_opt_struct_aref)(rb_execution_context_t *, rb_control_frame_t *);
-    const rb_iseq_t *iseq = rb_method_for_self_aref(name, off, rb_vm_opt_struct_aref);
+    const rb_iseq_t *iseq = rb_method_for_self_aref(name, off, &struct_aref_builtin);
+    iseq->body->builtin_inline_p = true;
 
     rb_add_method_iseq(nstr, SYM2ID(name), iseq, NULL, METHOD_VISI_PUBLIC);
 }
@@ -293,8 +332,7 @@ define_aref_method(VALUE nstr, VALUE name, VALUE off)
 static void
 define_aset_method(VALUE nstr, VALUE name, VALUE off)
 {
-    rb_control_frame_t *FUNC_FASTCALL(rb_vm_opt_struct_aset)(rb_execution_context_t *, rb_control_frame_t *);
-    const rb_iseq_t *iseq = rb_method_for_self_aset(name, off, rb_vm_opt_struct_aset);
+    const rb_iseq_t *iseq = rb_method_for_self_aset(name, off, &struct_aset_builtin);
 
     rb_add_method_iseq(nstr, SYM2ID(name), iseq, NULL, METHOD_VISI_PUBLIC);
 }
@@ -310,24 +348,15 @@ rb_struct_s_inspect(VALUE klass)
 }
 
 static VALUE
-struct_new_kw(int argc, const VALUE *argv, VALUE klass)
-{
-    return rb_class_new_instance_kw(argc, argv, klass, RB_PASS_CALLED_KEYWORDS);
-}
-
-static VALUE
-setup_struct(VALUE nstr, VALUE members, int keyword_init)
+setup_struct(VALUE nstr, VALUE members)
 {
     long i, len;
-    VALUE (*new_func)(int, const VALUE *, VALUE) = rb_class_new_instance;
-
-    if (keyword_init) new_func = struct_new_kw;
 
     members = struct_set_members(nstr, members);
 
     rb_define_alloc_func(nstr, struct_alloc);
-    rb_define_singleton_method(nstr, "new", new_func, -1);
-    rb_define_singleton_method(nstr, "[]", new_func, -1);
+    rb_define_singleton_method(nstr, "new", rb_class_new_instance_pass_kw, -1);
+    rb_define_singleton_method(nstr, "[]", rb_class_new_instance_pass_kw, -1);
     rb_define_singleton_method(nstr, "members", rb_struct_s_members_m, 0);
     rb_define_singleton_method(nstr, "inspect", rb_struct_s_inspect, 0);
     len = RARRAY_LEN(members);
@@ -359,9 +388,10 @@ struct_make_members_list(va_list ar)
 {
     char *mem;
     VALUE ary, list = rb_ident_hash_new();
-    st_table *tbl = RHASH_TBL(list);
+    st_table *tbl = RHASH_TBL_RAW(list);
 
     RBASIC_CLEAR_CLASS(list);
+    OBJ_WB_UNPROTECT(list);
     while ((mem = va_arg(ar, char*)) != 0) {
 	VALUE sym = rb_sym_intern_ascii_cstr(mem);
 	if (st_insert(tbl, sym, Qtrue)) {
@@ -442,7 +472,7 @@ rb_struct_define(const char *name, ...)
 
     if (!name) st = anonymous_struct(rb_cStruct);
     else st = new_struct(rb_str_new2(name), rb_cStruct);
-    return setup_struct(st, ary, 0);
+    return setup_struct(st, ary);
 }
 
 VALUE
@@ -455,7 +485,7 @@ rb_struct_define_under(VALUE outer, const char *name, ...)
     ary = struct_make_members_list(ar);
     va_end(ar);
 
-    return setup_struct(rb_define_class_under(outer, name, rb_cStruct), ary, 0);
+    return setup_struct(rb_define_class_under(outer, name, rb_cStruct), ary);
 }
 
 /*
@@ -554,7 +584,8 @@ rb_struct_s_def(int argc, VALUE *argv, VALUE klass)
 
     rest = rb_ident_hash_new();
     RBASIC_CLEAR_CLASS(rest);
-    tbl = RHASH_TBL(rest);
+    OBJ_WB_UNPROTECT(rest);
+    tbl = RHASH_TBL_RAW(rest);
     for (i=0; i<argc; i++) {
 	VALUE mem = rb_to_symbol(argv[i]);
         if (rb_is_attrset_sym(mem)) {
@@ -574,7 +605,7 @@ rb_struct_s_def(int argc, VALUE *argv, VALUE klass)
     else {
 	st = new_struct(name, klass);
     }
-    setup_struct(st, rest, (int)keyword_init);
+    setup_struct(st, rest);
     rb_ivar_set(st, id_keyword_init, keyword_init);
     if (rb_block_given_p()) {
         rb_mod_module_eval(0, 0, st);
@@ -661,7 +692,9 @@ rb_struct_initialize_m(int argc, const VALUE *argv, VALUE self)
 VALUE
 rb_struct_initialize(VALUE self, VALUE values)
 {
-    return rb_struct_initialize_m(RARRAY_LENINT(values), RARRAY_CONST_PTR(values), self);
+    rb_struct_initialize_m(RARRAY_LENINT(values), RARRAY_CONST_PTR(values), self);
+    RB_GC_GUARD(values);
+    return Qnil;
 }
 
 static VALUE *
@@ -814,7 +847,7 @@ rb_struct_each_pair(VALUE s)
 
     RETURN_SIZED_ENUMERATOR(s, 0, 0, struct_enum_size);
     members = rb_struct_members(s);
-    if (rb_block_arity() > 1) {
+    if (rb_block_pair_yield_optimizable()) {
 	for (i=0; i<RSTRUCT_LEN(s); i++) {
 	    VALUE key = rb_ary_entry(members, i);
 	    VALUE value = RSTRUCT_GET(s, i);
@@ -1023,7 +1056,6 @@ rb_struct_pos(VALUE s, VALUE *name)
     }
 }
 
-NORETURN(static void invalid_struct_pos(VALUE s, VALUE idx));
 static void
 invalid_struct_pos(VALUE s, VALUE idx)
 {
@@ -1298,18 +1330,21 @@ rb_struct_size(VALUE s)
 
 /*
  * call-seq:
- *   struct.dig(key, ...)              -> object
+ *   struct.dig(key, *identifiers) -> object
  *
- * Extracts the nested value specified by the sequence of +key+
- * objects by calling +dig+ at each step, returning +nil+ if any
- * intermediate step is +nil+.
+ * Finds and returns the object in nested objects
+ * that is specified by +key+ and +identifiers+.
+ * The nested objects may be instances of various classes.
+ * See {Dig Methods}[rdoc-ref:doc/dig_methods.rdoc].
  *
+ * Examples:
  *   Foo = Struct.new(:a)
  *   f = Foo.new(Foo.new({b: [1, 2, 3]}))
- *
- *   f.dig(:a, :a, :b, 0)    # => 1
- *   f.dig(:b, 0)            # => nil
- *   f.dig(:a, :a, :b, :c)   # TypeError: no implicit conversion of Symbol into Integer
+ *   f.dig(:a) # => #<struct Foo a={:b=>[1, 2, 3]}>
+ *   f.dig(:a, :a) # => {:b=>[1, 2, 3]}
+ *   f.dig(:a, :a, :b) # => [1, 2, 3]
+ *   f.dig(:a, :a, :b, 0) # => 1
+ *   f.dig(:b, 0) # => nil
  */
 
 static VALUE
